/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcmplayerjob.h"

#include <QByteArray>
#include <QStringList>

#include <KLocale>

KDCMPlayerJob::KDCMPlayerJob( QObject *parent )
	: KDCJob( parent )
{
	p_aid = -1;
	p_curSec = 0;

	p_process = new KProcess( this );
	p_process->setOutputChannelMode( KProcess::MergedChannels );
	connect( p_process, SIGNAL( readyReadStandardOutput() ), this, SLOT( readAllOutput() ) );
	connect( p_process, SIGNAL( finished( int, QProcess::ExitStatus ) ), this, SLOT( on_finished( int, QProcess::ExitStatus ) ) );
	connect( p_process, SIGNAL( started() ), this, SLOT( on_jobStarted() ) );
}

void KDCMPlayerJob::setFile( const QString& file )
{
	p_files.clear();
	p_files << file;
}

void KDCMPlayerJob::setFiles( const QStringList& files )
{
	p_files = files;
}

void KDCMPlayerJob::setAudioID( int id )
{
	if ( p_aid != id )
	{
		qDebug( "KDCMPlayerJob::setAudioID( %d )", id );
		p_aid = id;
		if ( isRunning() )
			restart();
	}
}

void KDCMPlayerJob::setWindowID( WId id )
{
	p_winID = id;
}

void KDCMPlayerJob::setVideoDriver( const QString& driver )
{
	if ( p_driver != driver )
	{
		qDebug( QString( "KDCMPlayerJob::setVideoDriver( %1 )" ).arg( driver ).toUtf8() );
		p_driver = driver;
		if ( isRunning() )
			restart();
	}
}

KDCMPlayerJob::~KDCMPlayerJob()
{
	if ( p_process )
	{
		p_process->disconnect();
		if( p_process->state() == QProcess::Running )
			p_process->close();
		delete p_process;
	}
}

void KDCMPlayerJob::start()
{
	if ( isRunning() )
		return;

	qDebug( "KDCMPlayerJob::start()" );
	p_id = 1;
	QStringList arg;
	arg << "-slave" << "-quiet" << "-noautosub" << "-osdlevel" << "0";
	arg << "-vo" << p_driver << "-wid" << QString::number( p_winID );
	if ( p_aid >= 0 )
		arg << "-aid" << QString::number( p_aid );
	for ( int i=0; i<p_files.size(); ++i )
		arg << p_files.at( i );

	p_process->setProgram( "mplayer", arg );
	p_process->start();

	p_timerID = startTimer( 1000 );
}

void KDCMPlayerJob::startAndSeek( int ss )
{
	if ( isRunning() )
		return;

	qDebug( "KDCMPlayerJob::startAndSeek()" );
	p_id = 1;
	QStringList arg;
	arg << "-slave" << "-quiet" << "-noautosub" << "-osdlevel" << "0";
	arg << "-vo" << p_driver << "-wid" << QString::number( p_winID );
	arg << "-ss" << QString::number( ss );
	if ( p_aid >= 0 )
		arg << "-aid" << QString::number( p_aid );
	for ( int i=0; i<p_files.size(); ++i )
		arg << p_files.at( i );

	p_process->setProgram( "mplayer", arg );
	p_process->start();

	p_timerID = startTimer( 1000 );
}

void KDCMPlayerJob::pause()
{
	if ( isRunning() && !flag( 1 ) && !flag( 2 ) )
	{
		qDebug( "Pause" );
		p_process->write( QString( "pause\n" ).toUtf8() );
		setFlag( 1, TRUE );
	}
}

void KDCMPlayerJob::unpause()
{
	if( isRunning() && flag( 1 ) && !flag( 2 ) )
	{
		qDebug( "Unpause" );
		p_process->write( QString( "pause\n" ).toUtf8() );
		setFlag( 1, FALSE );
	}
}

void KDCMPlayerJob::stop()
{
	if( !isRunning() )
		return;

	qDebug( "KDCMPlayerJob::stop" );
	setFlag( 1, FALSE );
	setFlag( 2, FALSE );

	killTimer( p_timerID );
	p_process->write( QString( "quit\n" ).toUtf8() );
	if ( !p_process->waitForFinished() )
		p_process->close();
}

void KDCMPlayerJob::on_finished( int code, QProcess::ExitStatus status )
{
	qDebug( "KDCMPlayerJob::finished()::codec = %d, status = %d", code, status );
	if ( status == QProcess::CrashExit )
		setError( i18n("There was an error during process. Error codec is %1").arg( code ) );
	setFlag( 1, FALSE );
	setFlag( 2, FALSE );
	killTimer( p_timerID );
	emit finished( this );
}

void KDCMPlayerJob::readAllOutput()
{
	while( p_process->canReadLine() )
	{
		QByteArray buffer( p_process->readLine() );
		if( buffer.startsWith( "ANS_TIME" ) )
		{
			QByteArray tmp;
			int idx = 18;
			char byte = buffer.at( idx );
			while ( byte != '\n' )
			{
				tmp.append( byte );
				++idx;
				byte = buffer.at( idx );
			}
			p_curSec = qRound( tmp.toDouble() - 1.0 );
			emit progress( p_curSec );
		}
	}
}

void KDCMPlayerJob::timerEvent( QTimerEvent *event )
{
	if ( isRunning() && !flag( 1 ) && !flag( 2 ) )
		p_process->write( QString( "get_time_pos\n" ).toUtf8() );
}

void KDCMPlayerJob::setSeek( int value )
{
	if ( isRunning() && !flag( 1 ) && !flag( 2 ) )
	{
		p_process->write( QString( "seek " + QString::number( value ) + " 2\n" ).toUtf8() );
		setFlag( 1, FALSE );
		setFlag( 2, FALSE );
		p_curSec = value;
		emit progress( value );
	}
}

int KDCMPlayerJob::currentTime() const
{
	return p_curSec;
}

void KDCMPlayerJob::forcePause()
{
	if ( isRunning() )
	{
		if ( !flag( 2 ) ) // we need pause
		{
			qDebug( "Force Pause" );
			pause();
			setFlag( 2, TRUE );
		}
		else // we need unpause
		{
			qDebug( "Force Unpause" );
			setFlag( 2, FALSE );
			unpause();
		}
	}
}

bool KDCMPlayerJob::isRunning()
{
	if ( p_process && p_process->state() == QProcess::Running )
		return TRUE;
	else
		return FALSE;
}

void KDCMPlayerJob::on_jobStarted()
{
	emit jobStarted( this );
}

void KDCMPlayerJob::playNext( int id )
{
	if ( !p_process )
		return;

	p_process->write( QString( "pt_step %1\n" ).arg( qAbs( id ) ).toUtf8() );

	if ( flag( 2 ) )
		p_process->write( QString( "pause\n" ).toUtf8() );
}

void KDCMPlayerJob::playPrev( int id )
{
	if ( !p_process )
		return;

	p_process->write( QString( "pt_step -%1\n" ).arg( qAbs( id ) ).toUtf8() );

	if ( flag( 2 ) )
		p_process->write( QString( "pause\n" ).toUtf8() );
}

int KDCMPlayerJob::currentMovie() const
{
	return p_id;
}

void KDCMPlayerJob::setCurrentMovie( int id )
{
	p_id = id;
}

void KDCMPlayerJob::restart()
{
	if ( !isRunning() )
		return;
	qDebug( "KDCMPlayerJob::restart()::begin" );
	p_process->blockSignals( TRUE );
	int id = currentMovie();
	int pos = currentTime();
	killTimer( p_timerID );
	p_process->write( QString( "quit\n" ).toUtf8() );
	if ( !p_process->waitForFinished() )
		p_process->close();
	start();
	setCurrentMovie( id );
	p_process->waitForStarted();
	if ( id > 1 )
		playNext( id - 1 );
	p_process->write( QString( "seek " + QString::number( pos ) + " 2\n" ).toUtf8() );
	if ( flag( 2 ) )
		p_process->write( QString( "pause\n" ).toUtf8() );
	p_process->blockSignals( FALSE );
	qDebug( "KDCMPlayerJob::restart()::end" );
}

bool KDCMPlayerJob::isPause()
{
	return flag( 2 );
}
