/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcsubtitlelist.h"

#include "../../kdcfilefactory.h"
#include "../../kdcnamespace.h"
#include "../../lib/kdclib.h"

#include <QStringList>
#include <QHeaderView>
#include <QTreeWidgetItem>
#include <QUrl>
#include <QMimeData>
#include <QDrag>

#include <KLocale>

KDCSubtitleList::KDCSubtitleList( QWidget *parent )
	: QTreeWidget( parent )
{
	setAlternatingRowColors( FALSE );
	setColumnCount( 4 );
	setSelectionBehavior( QAbstractItemView::SelectRows );
	setAcceptDrops( TRUE );
	setDropIndicatorShown( TRUE );
	setRootIsDecorated( FALSE );

	setColumnWidth( 0, qMax( 0, width() - 305 ) );
	setColumnWidth( 1, 160 );
	setColumnWidth( 2, 60 );
	setColumnWidth( 3, 80 );

	QStringList headers;
	headers << i18n("File Name") << i18n("Text Codec") << i18n("Size") << i18n("Fps");
	setHeaderLabels( headers );
	header()->setDefaultAlignment( Qt::AlignHCenter | Qt::AlignVCenter );

	p_fpsDelegate = new KDCFpsDelegate;
	connect( p_fpsDelegate, SIGNAL( fpsChanged( double ) ), this, SLOT( on_fpsChanged( double ) ) );
	connect( p_fpsDelegate, SIGNAL( closeEditor( QWidget *, QAbstractItemDelegate::EndEditHint ) ), this, SLOT( on_closeEditor( QWidget *, QAbstractItemDelegate::EndEditHint ) ) );
	p_sizeDelegate = new KDCTextSizeDelegate;
	connect( p_sizeDelegate, SIGNAL( sizeChanged( int ) ), this, SLOT( on_sizeChanged( int ) ) );
	connect( p_sizeDelegate, SIGNAL( closeEditor( QWidget *, QAbstractItemDelegate::EndEditHint ) ), this, SLOT( on_closeEditor( QWidget *, QAbstractItemDelegate::EndEditHint ) ) );
	p_codecDelegate = new KDCTextCodecDelegate;
	connect( p_codecDelegate, SIGNAL( codecChanged( const QString& ) ), this, SLOT( on_codecChanged( const QString& ) ) );
	connect( p_codecDelegate, SIGNAL( closeEditor( QWidget *, QAbstractItemDelegate::EndEditHint ) ), this, SLOT( on_closeEditor( QWidget *, QAbstractItemDelegate::EndEditHint ) ) );

	connect( this, SIGNAL( itemDoubleClicked( QTreeWidgetItem *, int ) ), this, SLOT( openEditor( QTreeWidgetItem *, int ) ) );
	connect( this, SIGNAL( currentItemChanged( QTreeWidgetItem *, QTreeWidgetItem * ) ), this, SLOT( on_currentItemChanged( QTreeWidgetItem *, QTreeWidgetItem * ) ) );

	clean();
}

KDCSubtitleList::~KDCSubtitleList()
{
	delete p_fpsDelegate;
	delete p_codecDelegate;
	delete p_sizeDelegate;
}

void KDCSubtitleList::clean()
{
	qDebug( "KDCSubtitleList::clean()" );
	invisibleRootItem()->takeChildren();
	p_movie = NULL;
	p_id = -1;
}

void KDCSubtitleList::setCurrentMovie( KDCMovie *movie )
{
	if ( p_movie != movie )
	{
		qDebug( "KDCSubtitleList::setCurrentMovie( 1 )" );
		clean();
		p_movie = movie;
		for ( int i=0; i<movie->subtitleStreamGroup()->count(); ++i )
			addTopLevelItem( movie->subtitleStreamGroup()->stream( i ) );
	}
}

void KDCSubtitleList::refresh()
{
	if ( p_movie )
	{
		blockSignals( TRUE );
		QTreeWidgetItem *item = currentItem();
		invisibleRootItem()->takeChildren();
		for ( int i=0; i<p_movie->subtitleStreamGroup()->count(); ++i )
			addTopLevelItem( p_movie->subtitleStreamGroup()->stream( i ) );
		setCurrentItem( item );
		p_id = indexOfTopLevelItem( item );
		blockSignals( FALSE );
	}
}

void KDCSubtitleList::openEditor( QTreeWidgetItem *item, int column )
{
	if ( column != 0 )
	{
		switch ( column )
		{
			case 1:
			{
				setItemDelegateForColumn( 1, p_codecDelegate );
				p_codecDelegate->setCurrentData( p_movie->subtitleStreamGroup()->stream( p_id )->textCodec() );
				break;
			}
			case 2:
			{
				setItemDelegateForColumn( 2, p_sizeDelegate );
				p_sizeDelegate->setCurrentData( p_movie->subtitleStreamGroup()->stream( p_id )->textSize() );
				break;
			}
			case 3:
			{
				setItemDelegateForColumn( 3, p_fpsDelegate );
				p_fpsDelegate->setCurrentData( p_movie->subtitleStreamGroup()->stream( p_id )->fps() );
				break;
			}
		}
		item->setFlags( item->flags() | Qt::ItemIsEditable );
		editItem( item, column );
		qDebug( "KDCSubtitleList::openEditor()::end" );
	}
}

KDCSubtitleStream* KDCSubtitleList::currentSubtitleStream() const
{
	return p_movie->subtitleStreamGroup()->stream( p_id );
}

void KDCSubtitleList::on_currentItemChanged( QTreeWidgetItem *current, QTreeWidgetItem *previous )
{
	qDebug( "KDCSubtitleList::currentItemChanged()::current = %d", KDCLib::isNull( current ) );
	if ( current )
	{
		KDCSubtitleStream *sub = static_cast<KDCSubtitleStream*>( current );
		if ( sub != p_movie->subtitleStreamGroup()->stream( p_id ) )
		{
			p_id = indexOfTopLevelItem( current );
			emit currentSubtitleStreamChanged( sub );
		}
	}
	else
	{
		p_id = -1;
		emit currentSubtitleStreamChanged( NULL );
	}
}

void KDCSubtitleList::on_closeEditor( QWidget *w, QAbstractItemDelegate::EndEditHint h )
{
	qDebug( "KDCSubtitleEditor::closeEditor()" );
	currentItem()->setFlags( Qt::ItemIsSelectable | Qt::ItemIsEnabled );
}

void KDCSubtitleList::on_fpsChanged( double val )
{
	p_movie->subtitleStreamGroup()->stream( p_id )->setFps( val );
}

void KDCSubtitleList::on_sizeChanged( int val )
{
	p_movie->subtitleStreamGroup()->stream( p_id )->setTextSize( val );
}

void KDCSubtitleList::on_codecChanged( const QString& codec )
{
	p_movie->subtitleStreamGroup()->stream( p_id )->setTextCodec( codec );
}

QStringList KDCSubtitleList::mimeTypes() const
{
	QStringList qstrList;
	qstrList.append("text/uri-list");
	return qstrList;
}

Qt::DropActions KDCSubtitleList::supportedDropActions() const
{
	return Qt::CopyAction;
}

void KDCSubtitleList::mouseMoveEvent( QMouseEvent *event )
{
	if ( !( event->buttons() & Qt::LeftButton ) )
		return;

	qDebug( "KDCSubtitleList::mouseMoveEvent()::currentItem = %d", KDCLib::isNull( currentItem() ) );
	if ( currentItem() )
	{
		QList<QUrl> list;
		list.append( QUrl( QString::number( indexOfTopLevelItem( currentItem() ) ) ) );
		QMimeData *mData = new QMimeData;
		QDrag *drag = new QDrag( this );
		mData->setUrls( list );
		drag->setMimeData( mData );

		qDebug( "KDCSubtitleList::mouseMoveEvent()::start draging" );
		drag->start( Qt::CopyAction );
	}
}

bool KDCSubtitleList::dropMimeData( QTreeWidgetItem *parent, int index, const QMimeData *data, Qt::DropAction action )
{
	QList<QUrl> list = data->urls();
	int pos = list.first().toString().toInt();
	qDebug( "KDCSubtitleList::dropMimeData:: pos = %d", pos );
	if ( parent )
	{
		int newPos = indexOfTopLevelItem( parent );
		if ( newPos == pos )
			return FALSE;

		blockSignals( TRUE );
		QTreeWidgetItem *item = takeTopLevelItem( pos );
		insertTopLevelItem( newPos, item );
		p_movie->subtitleStreamGroup()->move( pos, newPos );
		setCurrentItem( item );
		blockSignals( FALSE );
		return TRUE;
	}
	else
		return FALSE;
}
