/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcmplayerwidget.h"

#include "../kdcconfig.h"

#include <QLayout>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QPalette>
#include <QStringList>
#include <QTime>
#include <QVariant>

#include <KLocale>
#include <KIcon>

KDCMPlayerWidget::KDCMPlayerWidget( QWidget *parent )
	: QWidget( parent )
{
	p_mainWindow = new QWidget;
	p_mainWindow->setAttribute( Qt::WA_PaintOnScreen );
	p_mainWindow->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed ) );
	QPalette p = p_mainWindow->palette();
	p.setColor( QPalette::Window, Qt::black );
	p_mainWindow->setPalette( p );

	p_subWindow = new QWidget( p_mainWindow );
	p_subWindow->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed ) );
	p_subWindow->setAttribute( Qt::WA_PaintOnScreen );

	QToolButton *p_play = new QToolButton;
	p_play->setIcon( KIcon("media-playback-start") );
	p_play->setMaximumSize( 50, p_play->height() );
	connect( p_play, SIGNAL( clicked() ), this, SIGNAL( playClicked() ) );
	p_buttons.insert( PLAY, p_play );

	QToolButton *p_stop = new QToolButton;
	p_stop->setIcon( KIcon("media-playback-stop") );
	p_stop->setMaximumSize( 50, p_stop->height() );
	connect( p_stop, SIGNAL( clicked() ), this, SIGNAL( stopClicked() ) );
	p_buttons.insert( STOP, p_stop );

	QToolButton *p_pause = new QToolButton;
	p_pause->setIcon( KIcon("media-playback-pause") );
	p_pause->setMaximumSize( 50, p_pause->height() );
	connect( p_pause, SIGNAL( clicked() ), this, SIGNAL( pauseClicked() ) );
	p_buttons.insert( PAUSE, p_pause );

	p_timeSeek = new QSlider( Qt::Horizontal );
	connect( p_timeSeek, SIGNAL( sliderMoved( int ) ), this, SIGNAL( timeSeekChanged( int ) ) );

	p_time = new QLabel( "00:00:00" );

	p_panel = new QWidget;
	QHBoxLayout *hLayout = new QHBoxLayout( p_panel );
	hLayout->addWidget( p_play );
	hLayout->addWidget( p_stop );
	hLayout->addWidget( p_pause );
	hLayout->addWidget( p_timeSeek );
	hLayout->addWidget( p_time );

	delete layout();
	QVBoxLayout *vLayout = new QVBoxLayout( this );
	vLayout->addWidget( p_mainWindow );
	vLayout->addWidget( p_panel );
	vLayout->addStretch();
	vLayout->setSizeConstraint( QLayout::SetFixedSize );

	p_popupMenu = new KMenu;

	p_aid = new KSelectAction( i18n("Select audio track"), this );
	p_popupMenu->addAction( p_aid );
	connect( p_aid, SIGNAL( triggered( QAction * ) ), this, SLOT( audioIDTriggered( QAction * ) ) );

	p_vo = new KSelectAction( i18n("Select video driver"), this );
	QStringList drivers;
	drivers << "xv" << "x11" << "gl" << "gl2" << "sdl" << "directfb";
	p_vo->setItems( drivers );
	p_vo->setCurrentAction( KDCConfig::driver() );
	connect( p_vo, SIGNAL( triggered( const QString& ) ), this, SIGNAL( videoDriverChanged( const QString& ) ) );
	p_popupMenu->addAction( p_vo );
}

KDCMPlayerWidget::~KDCMPlayerWidget()
{
	delete p_popupMenu;
}

WId KDCMPlayerWidget::windowID() const
{
	return p_subWindow->winId();
}

void KDCMPlayerWidget::showControlButton( ControlButton button, bool show )
{
	p_buttons.value( button )->setVisible( show );
}

void KDCMPlayerWidget::showTimeLabel( bool show )
{
	p_time->setVisible( show );
}

void KDCMPlayerWidget::showTimeSlider( bool show )
{
	p_timeSeek->setVisible( show );
}

void KDCMPlayerWidget::resizeEvent( QResizeEvent *event )
{
	setMovieProperties( p_width, p_height, p_aspect );
}

QWidget* KDCMPlayerWidget::controlPanel() const
{
	return p_panel;
}

void KDCMPlayerWidget::setWindowSize( int width, int height )
{
	setMinimumSize( width, height );
	p_mainWindow->setMinimumSize( width, height );
	p_subWindow->setMinimumSize( width, height );
}

void KDCMPlayerWidget::setMovieProperties( int width, int height, double aspect, int length )
{
	p_width = width;
	p_height = height;
	p_aspect = aspect;
	p_timeSeek->setMaximum( length );
	p_timeSeek->setValue( 0 );

	double scale_w = (double)p_mainWindow->width() / (double)p_width;
	int new_height_w = qRound( scale_w * p_height );
	if ( new_height_w <= p_mainWindow->height() ) // width
	{
		//int size = qBound( 0, qRound( p_mainWindow->width()/aspect ), p_mainWindow->height() );
		p_subWindow->setMinimumSize( p_mainWindow->width(), new_height_w );
		p_subWindow->setMaximumSize( p_mainWindow->width(), new_height_w );
		p_subWindow->move( 0, qRound( qAbs( (double)new_height_w - (double)p_mainWindow->height() ) * 0.5 ) );
	}
	else // height
	{
		int size = qBound( 0, qRound( p_mainWindow->height()*aspect ), p_mainWindow->width() );
		p_subWindow->setMinimumSize( size, p_mainWindow->height() );
		p_subWindow->setMaximumSize( size, p_mainWindow->height() );
		p_subWindow->move( qRound( qAbs( (double)size - (double)p_mainWindow->width() ) * 0.5 ), 0 );
	}
}

void KDCMPlayerWidget::contextMenuEvent( QContextMenuEvent *event )
{
	p_popupMenu->exec( event->globalPos() );
}

QString KDCMPlayerWidget::videoDriver()
{
	return p_vo->currentText();
}

int KDCMPlayerWidget::audioID()
{
	QAction *action = p_aid->currentAction();
	if ( action )
		return action->data().toInt();
	else
		return -1;
}

void KDCMPlayerWidget::setAudioTracks( const QList<int>& list )
{
	p_aid->clear();

	KAction *action;
	for ( int i=0; i<list.size(); ++i )
	{
		action = p_aid->addAction( QString::number( i + 1 ) );
		action->setData( QVariant( list.at( i ) ) );
	}
	p_aid->setCurrentItem( 0 );
}

void KDCMPlayerWidget::setTimeSeek( int value )
{
	p_timeSeek->setValue( value );
	if ( p_time->isVisible() )
	{
		QTime time;
		p_time->setText( time.addSecs( value ).toString( "hh:mm:ss" ) );
	}
}

int KDCMPlayerWidget::currentPos() const
{
	return p_timeSeek->value();
}

void KDCMPlayerWidget::audioIDTriggered( QAction *action )
{
	emit audioIDChanged( action->data().toInt() );
}
